package uoc.tfc;


import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.DriverManager;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.hp.hpl.jena.db.DBConnection;
import com.hp.hpl.jena.db.IDBConnection;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.rdf.model.InfModel;
import com.hp.hpl.jena.rdf.model.Literal;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.ModelMaker;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.reasoner.ValidityReport;
import com.hp.hpl.jena.reasoner.ValidityReport.Report;
import com.hp.hpl.jena.shared.AlreadyExistsException;
import com.hp.hpl.jena.shared.DoesNotExistException;
import com.hp.hpl.jena.util.PrintUtil;


public class DBModel {

	private ModelMaker maker = null;
	private Model m = null;
	private InfModel iM = null;
	private IDBConnection conn = null;
	private Connection connection = null;	 
	private static final String ONTOLOGY_NAME = "cine_ontology";
	private static final String ONTOLOGY_FILE = "cine.rdfs";		
	private static final String MODEL_NAME = "Modelo_de_Cine";
	private static final String RESOURCE_URI = "http://dbpedia.org/resource/";
	private static final String DB_TYPE = "Derby";
	private static final String RDF = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
	private static final String ONTOLOGY_URI = "http://www.sfernandezve.org/cine#";  
	private static final String PREFIX = 
			"PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#> " +
			"PREFIX : <http://dbpedia.org/resource/> " +
			"PREFIX dbpedia2: <http://dbpedia.org/property/> " +
			"PREFIX dbpedia1: <http://dbpedia.org/ontology/> ";

	public DBModel() {
		
		String DRIVER = "org.apache.derby.jdbc.EmbeddedDriver";
		String protocol = "jdbc:derby:";
		String dbURL = "C:\\DevApp\\triplestore";
		String create = ";create=true";

		try {
			Class.forName(DRIVER).newInstance();
			connection = DriverManager.getConnection(protocol + dbURL + create);
			conn = new DBConnection(connection,DB_TYPE);
		} catch (Exception e) { 
			e.printStackTrace(); 
		}
		maker = ModelFactory.createModelRDBMaker(conn);
				
	}
	
	public Iterator<String> listaObrasCargadas(){

		// Consulta las Obras descargadas en el  modelo cargado en Derby.
		
		List<String> lista = consultaObrasDescargadas(iM);
		Iterator<String> itr = lista.iterator(); 
		return itr;
	}
	
	public Iterator<String> listaPeliculasCargadas(){

		// Consulta las peliculas descargadas en el  modelo cargado en Derby.
		
		List<String> lista = consultaPeliculasDescargadas(iM);
		Iterator<String> itr = lista.iterator(); 
		return itr;
	}
	
	public Iterator<String> listaDirectoresCargados(){

		// Consulta los directores descargados en el  modelo cargado en Derby.
		
		List<String> lista = consultaDirectoresDescargados(iM);
		Iterator<String> itr = lista.iterator(); 
		return itr;
	}
	
	public Iterator<String> listaProductoresCargados(){

		// Consulta los productores descargados en el  modelo cargado en Derby.
		
		List<String> lista = consultaProductoresDescargados(iM);
		Iterator<String> itr = lista.iterator(); 
		return itr;
	}
	
	public Iterator<String> listaProtagonistasCargados(){

		// Consulta los protagonistas descargados en el  modelo cargado en Derby.
		
		List<String> lista = consultaProtagonistasDescargados(iM);
		Iterator<String> itr = lista.iterator(); 
		return itr;
	}

	public void cargaModelos() {
		
		List<String> listaDirectores  = new ArrayList<String>();
		listaDirectores.add("Steven_Spielberg");
		listaDirectores.add("George_Lucas");
		listaDirectores.add("David_Lynch");
		listaDirectores.add("Tim_Burton");
		listaDirectores.add("Stanley_Kubrick");
		listaDirectores.add("Clint_Eastwood");
		listaDirectores.add("Ridley_Scott");
	  
		try {
			 cargaOntologia();
		} catch (IOException e) {
			e.printStackTrace();
		}
		cargaRecursos(listaDirectores);
		
		if (iM != null) {
			iM.reset();
			iM = null;
		}
		
		verificarModelos();
	
	}
		
	public Boolean verificarModelos() {
		if ( maker.hasModel(ONTOLOGY_NAME)  & maker.hasModel(MODEL_NAME)){
			if (iM == null) { 
				iM = creaModeloInferencia();
			} 
			return true;
		} else {
			return false;
		}
	}
	
	
	public void cargaOntologia() throws IOException {
	
		// Extraer y cargar en la base de datos la ontologia que hemos creado. 
		
		if ( !maker.hasModel(ONTOLOGY_NAME) ) {
			// Crea el modelo de la ontologia en Derby la primera vez.
			m = maker.createModel(ONTOLOGY_NAME);
		} else {
			// Borra el modelo existente de la ontologia en Derby y lo vuelve a crear si ya esta creado.
			maker.removeModel(ONTOLOGY_NAME);
			// Crea el modelo de la ontologia en Derby.
			m = maker.createModel(ONTOLOGY_NAME);
		}
			
		// Utilizamos el class loader para encontrar el fichero que contiene la ontologia
		// El fichero se encuentra en el directorio /WEB-INF/classes
		InputStream in = this.getClass().getClassLoader().getResourceAsStream(ONTOLOGY_FILE);
		if (in == null) {
		       throw new FileNotFoundException("Fichero no encontrado en classpath: "+ ONTOLOGY_FILE);
		}
		
		// Leer el contenido de la ontologia y hacerlo persistente en Derby.
	    m.read(in,null);
	    
	    // Para saber el contenido del modelo cargado descomentar las lineas siguientes.
		//   m.write(System.out);
		//   listarTodasSentencias(m, null, null, null);

	}

	
	public void cargaRecursos(List<String> listaDirectores) {
	
		// Abre el modelo indicado en la variable. Si el modelo no existe
		// en la base de datos, encontes lo crea. En caso contrario, lo borra y 
		// luego lo crea.
		// En cualquier caso, partimos de un modelo "m" vacio.
		
		
		try {
			m = maker.createModel(MODEL_NAME,true);	
		} catch (DoesNotExistException e) { 
			// Crea el modelo de los datos en Derby la primera vez.
			m = maker.createModel(MODEL_NAME,true);
		} catch (AlreadyExistsException e) { 
			// Borra el modelo existente de los datos en Derby y lo vuelve a crear si ya esta creado.
			maker.removeModel(MODEL_NAME);
			// Crea el modelo de los datos en Derby.
			m = maker.createModel(MODEL_NAME,true);
		} catch (Exception e) {
			e.printStackTrace(); 
		}
		
		// Para hacer el tipado de los recursos
		
		Property Type = m.createProperty(RDF,"type");
		Property Director = m.createProperty(ONTOLOGY_URI,"Director");
		Property Protagonista = m.createProperty(ONTOLOGY_URI,"Protagonista");
		Property Productor = m.createProperty(ONTOLOGY_URI,"Productor");
		Property Pelicula = m.createProperty(ONTOLOGY_URI,"Pelicula");
		
		// Definimos las propiedades de las clases
		
		Property esDirectorDe = m.createProperty(ONTOLOGY_URI,"esDirectorDe");
	    Property estaDirigidaPor = m.createProperty(ONTOLOGY_URI,"estaDirigidaPor");
	    Property esProtagonistaDe = m.createProperty(ONTOLOGY_URI,"esProtagonistaDe");
	    Property estaProtagonizadaPor = m.createProperty(ONTOLOGY_URI,"estaProtagonizadaPor");
	    Property esProductorDe = m.createProperty(ONTOLOGY_URI,"esProductorDe");
	    Property estaProducidaPor = m.createProperty(ONTOLOGY_URI,"estaProducidaPor");
	    
	    // Definimos los datos de las clases
	    
	    Property Nombre = m.createProperty(ONTOLOGY_URI,"Nombre");
	    Property Titulo = m.createProperty(ONTOLOGY_URI,"Titulo");
	    Property Sinopsis = m.createProperty(ONTOLOGY_URI,"Sinopsis");
	    
		
		// Extraer nombre de 5 peliculas de Dbpedia por director usando el servicio 
		// SPARQL Endpoint de DBpedia. 
	    // Crear instancia director.
				
		Iterator<String> itrLdirector = listaDirectores.iterator();
		while(itrLdirector.hasNext()) {
		    Resource RDirector = null;
			String director = itrLdirector.next();
			RDirector = m.createResource(RESOURCE_URI + director);
			RDirector.addProperty(Type,Director);
			RDirector.addProperty(Nombre, extractDbpediaNombre(RESOURCE_URI + director));
			
			Iterator<String> itrLpelicula = extractDbpediaPeliculas(director).iterator();
		
			// Crear instancia pelicula. Completar instancia director con sus peliculas.
		
			while(itrLpelicula.hasNext()) {			
                Resource RPelicula = null;
				String pelicula = itrLpelicula.next();
				RPelicula = m.createResource(pelicula);
				// RPelicula.addProperty(Type, Pelicula);
				RDirector.addProperty(esDirectorDe, RPelicula);
				RPelicula.addProperty(estaDirigidaPor, RDirector);
				RPelicula.addProperty(Titulo, extractDbpediaTitulo(pelicula));
				RPelicula.addProperty(Sinopsis, extractDbpediaSinopsis(pelicula));
				
				// Para la pelicula obtener protagonistas.
				// Crear instancia protagonista. Completar instancia pelicula con sus protagonistas.
				
				Iterator<String> itrLprotagonistas = extractDbpediaProtagonistas(pelicula).iterator();
				while(itrLprotagonistas.hasNext()) {		
				    Resource RProtagonista = null;
					String protagonista = itrLprotagonistas.next();
					RProtagonista = m.createResource(protagonista);
					RProtagonista.addProperty(Type, Protagonista);
					RPelicula.addProperty(estaProtagonizadaPor, RProtagonista);
					RProtagonista.addProperty(esProtagonistaDe, RPelicula);
					RProtagonista.addProperty(Nombre, extractDbpediaNombre(protagonista));
				}
				  
				// Para la pelicula obtener Productores.
				// Crear instancia productor. Completar instancia pelicula con sus productores.
				
				Iterator<String> itrLproductores = extractDbpediaProductores(pelicula).iterator();
				while(itrLproductores.hasNext()) {
					Resource RProductor = null;
					String productor = itrLproductores.next();
					RProductor = m.createResource(productor);
					RProductor.addProperty(Type, Productor);
					RPelicula.addProperty(estaProducidaPor, RProductor);
					RProductor.addProperty(esProductorDe, RPelicula);					
					RProductor.addProperty(Nombre, extractDbpediaNombre(productor));
				}
			}	
		}

		// Para saber el contenido del modelo cargado descomentar la siguientes lineas.
		
		    // listarTodasSentencias(m, null, null, null);
			// m.write(System.out);
		
		// Cerrar el modelo y liberar recursos
		m.close();
	}
	
	public InfModel creaModeloInferencia(){
		
		// Cargamos el documento con los datos de la base de datos
		Model datos = maker.getModel(MODEL_NAME);
		
		// Cargamos el documento con la ontologia de la base de datos
		Model ontologia = maker.getModel(ONTOLOGY_NAME);
		
		// Creamos el modelo de inferencia 
		InfModel modeloInferencia = ModelFactory.createRDFSModel(ontologia, datos);	
		
		// Para listar el contenido del modelo de inferencia descomentar la siguiente linea
		// modeloInferencia.write(System.out);
	     
		// Validamos el modelo
		ValidityReport vr = modeloInferencia.validate();
		
		if ( vr.isValid() == false ) {
			System.out.print(" La validacion del modelo de inferencia ha enctonrado errores.");
			for (Iterator<Report> i = vr.getReports(); i.hasNext(); ) {
				System.out.println(" - " + i.next());
			} 	
			
			return null;
			
		} else {
			
			System.out.print(" Validacion del modelo de inferencia OK !! .");
			return modeloInferencia;
		}  	
		
	}

	public List<String> consultaPeliculasDescargadas(InfModel iM){
		 List<String> lista = new ArrayList<String>();
		 String query = PREFIX + 
	    		   " SELECT distinct ?titulo "
	               + " WHERE {  "
	               + "   ?recurso rdf:type <http://www.sfernandezve.org/cine#Pelicula> .  "
	               + "   ?recurso <" +  ONTOLOGY_URI + "Titulo>  ?titulo .  " 
	               + " } ORDER BY ASC(?titulo) ";
	       
	       QueryExecution qe = QueryExecutionFactory.create(query, iM);

	       try {
	    	   ResultSet results = qe.execSelect();
	            // Para cada pelicula encontrada
	            while(results.hasNext()){
	                QuerySolution r = results.nextSolution();
	                RDFNode pelicula = r.get("titulo");
	                // Obtenemos titulo en formato String y lo aadimos a la lista
	                if ( !((Literal)pelicula).getLexicalForm().equals("Desconocido"))
	                   lista.add(((Literal)pelicula).getLexicalForm()) ;
	            }
	        } catch (Exception ex) {
	            ex.printStackTrace();
	        } finally {
				// Importante - liberar recursos despues de hacer la consulta
				qe.close();
			}
		
		return lista;
		
	}
	
	public List<String> consultaDirectoresDescargados(InfModel iM){
		 List<String> lista = new ArrayList<String>();
		 String query = PREFIX + 
	    		   " SELECT distinct ?nombre "
	               + " WHERE {  "
	               + "   ?recurso rdf:type <http://www.sfernandezve.org/cine#Director> .  "
	               + "   ?recurso <" +  ONTOLOGY_URI + "Nombre>  ?nombre .  " 
	               + " } ORDER BY ASC(?nombre) " ;
	       
	       QueryExecution qe = QueryExecutionFactory.create(query, iM);

	       try {
	    	   ResultSet results = qe.execSelect();
	            // Para cada director encontrado
	            while(results.hasNext()){
	                QuerySolution r = results.nextSolution();
	                RDFNode director = r.get("nombre");
	                // Obtenemos nombre en formato String y lo aadimos a la lista
	                if ( !((Literal)director).getLexicalForm().equals("Desconocido"))
	                   lista.add(((Literal)director).getLexicalForm()) ;
	            }
	        } catch (Exception ex) {
	            ex.printStackTrace();
	        } finally {
				// Importante - liberar recursos despues de hacer la consulta
				qe.close();
			}
		
		return lista;
		
	}
	
	public List<String> consultaProductoresDescargados(InfModel iM){
		 List<String> lista = new ArrayList<String>();
		 String query = PREFIX + 
	    		   " SELECT distinct ?nombre "
	               + " WHERE {  "
	               + "   ?recurso rdf:type <http://www.sfernandezve.org/cine#Productor> .  "
	               + "   ?recurso <" +  ONTOLOGY_URI + "Nombre>  ?nombre .  " 
	               + " } ORDER BY ASC(?nombre) " ;
	       
	       QueryExecution qe = QueryExecutionFactory.create(query, iM);

	       try {
	    	   ResultSet results = qe.execSelect();
	            // Para cada productor encontrado
	            while(results.hasNext()){
	                QuerySolution r = results.nextSolution();
	                RDFNode productor = r.get("nombre");
	                // Obtenemos nombre en formato String y lo aadimos a la lista
	                if ( !((Literal)productor).getLexicalForm().equals("Desconocido"))
	                   lista.add(((Literal)productor).getLexicalForm()) ;
	            }
	        } catch (Exception ex) {
	            ex.printStackTrace();
	        } finally {
				// Importante - liberar recursos despues de hacer la consulta
				qe.close();
			}
		
		return lista;
		
	}
	
	public List<String> consultaProtagonistasDescargados(InfModel iM){
		 List<String> lista = new ArrayList<String>();
		 String query = PREFIX + 
	    		   " SELECT distinct ?nombre "
	               + " WHERE {  "
	               + "   ?recurso rdf:type <http://www.sfernandezve.org/cine#Protagonista> .  "
	               + "   ?recurso <" +  ONTOLOGY_URI + "Nombre>  ?nombre .  " 
	               + " } ORDER BY ASC(?nombre) ";
	       
	       QueryExecution qe = QueryExecutionFactory.create(query, iM);

	       try {
	    	   ResultSet results = qe.execSelect();
	            // Para cada protagonista encontrado
	            while(results.hasNext()){
	                QuerySolution r = results.nextSolution();
	                RDFNode protagonista = r.get("nombre");
	                // Obtenemos nombre en formato String y lo aadimos a la lista
	                if ( !((Literal)protagonista).getLexicalForm().equals("Desconocido"))
	                   lista.add(((Literal)protagonista).getLexicalForm()) ;
	            }
	        } catch (Exception ex) {
	            ex.printStackTrace();
	        } finally {
				// Importante - liberar recursos despues de hacer la consulta
				qe.close();
			}
		
		return lista;
		
	}
	
	
	public List<String> consultaObrasDescargadas(InfModel iM){
		 List<String> lista = new ArrayList<String>();
		 String query = PREFIX + 
	    		   " SELECT distinct ?titulo "
	               + " WHERE {  "
	               + "   ?recurso rdf:type <http://www.sfernandezve.org/cine#Obra> .  "
	               + "   ?recurso <" +  ONTOLOGY_URI + "Titulo>  ?titulo .  " 
	               + "  } ORDER BY ASC(?titulo) ";
	       
	       QueryExecution qe = QueryExecutionFactory.create(query, iM);

	       try {
	    	   ResultSet results = qe.execSelect();
	            // Para cada pelicula encontrada
	            while(results.hasNext()){
	                QuerySolution r = results.nextSolution();
	                RDFNode pelicula = r.get("titulo");
	                // Obtenemos titulo en formato String y lo aadimos a la lista
	                if ( !((Literal)pelicula).getLexicalForm().equals("Desconocido"))
	                   lista.add(((Literal)pelicula).getLexicalForm()) ;
	            }
	        } catch (Exception ex) {
	            ex.printStackTrace();
	        } finally {
				// Importante - liberar recursos despues de hacer la consulta
				qe.close();
			}
		
		return lista;
		
	}
	
	public Pelicula consultaFichaPelicula(String nombrePelicula) {

		Pelicula pelicula = new Pelicula();
		pelicula.setTitulo(nombrePelicula);
		pelicula.setSinopsis(consultaSinopsis(iM, nombrePelicula));
		pelicula.setDirector(consultaDirector(iM, nombrePelicula));
		pelicula.setProductor(consultaProductor(iM, nombrePelicula));
		pelicula.setProtagonista(consultaProtagonista(iM, nombrePelicula));
		
		return pelicula;
	}
	
	public Director consultaFichaDirector(String nombreDirector) {
		
		Director director = new Director();
		director.setNombre(nombreDirector);
		director.setPeliculasDirigidas(consultaPeliculasDirigidas(iM, nombreDirector));
		director.setPeliculasProducidas(consultaPeliculasProducidas(iM, nombreDirector));
		director.setPeliculasProtagonizadas(consultaPeliculasProtagonizadas(iM, nombreDirector));
		
		return director;
	}
	
	public Productor consultaFichaProductor(String nombreProductor) {
		
		Productor productor = new Productor();
		productor.setNombre(nombreProductor);
		productor.setPeliculasDirigidas(consultaPeliculasDirigidas(iM, nombreProductor));
		productor.setPeliculasProducidas(consultaPeliculasProducidas(iM, nombreProductor));
		productor.setPeliculasProtagonizadas(consultaPeliculasProtagonizadas(iM, nombreProductor));
		
		return productor;
	}	
		
	public Protagonista consultaFichaProtagonista(String nombreProtagonista) {
		
		Protagonista protagonista = new Protagonista();
		protagonista.setNombre(nombreProtagonista);
		protagonista.setPeliculasDirigidas(consultaPeliculasDirigidas(iM, nombreProtagonista));
		protagonista.setPeliculasProducidas(consultaPeliculasProducidas(iM, nombreProtagonista));
		protagonista.setPeliculasProtagonizadas(consultaPeliculasProtagonizadas(iM, nombreProtagonista));
		
		return protagonista;
	}
	
	public String consultaSinopsis(InfModel iM, String titulo){
		 String resultado = "No dsiponible";
		 String query = PREFIX + 
	    		   " SELECT ?sinopsis "
	               + " WHERE {  "
	               + "   ?recurso rdf:type <http://www.sfernandezve.org/cine#Pelicula> .  "
				   + "   ?recurso <" +  ONTOLOGY_URI + "Titulo>\""  + titulo +   "\"  .  "
	               + "   ?recurso <" +  ONTOLOGY_URI + "Sinopsis>  ?sinopsis .  " 
	               + " } ";
	       
	       QueryExecution qe = QueryExecutionFactory.create(query, iM);
	       
	       try {
	    	   ResultSet results = qe.execSelect();
	            // Para cada director encontrado
	            if (results.hasNext()){
	                QuerySolution r = results.nextSolution();
	                RDFNode sinopsis = r.get("sinopsis");
	                // Obtenemos nombre en formato String y lo aadimos a la lista
	                resultado = (((Literal)sinopsis).getLexicalForm());
	            }
	        } catch (Exception ex) {
	            ex.printStackTrace();
	        } finally {
				// Importante - liberar recursos despues de hacer la consulta
				qe.close();
			}
		
		return resultado;
	}

	public List<String> consultaDirector(InfModel iM, String titulo){
		 List<String> lista = new ArrayList<String>();
		 String query = PREFIX + 
	    		   " SELECT distinct ?nombre "
	               + " WHERE {  "
	               + "   ?recurso rdf:type <http://www.sfernandezve.org/cine#Pelicula> .  "
				   + "   ?recurso <" +  ONTOLOGY_URI + "Titulo>\""  + titulo +   "\"  .  "
				   + "   ?director <" +  ONTOLOGY_URI + "esDirectorDe> ?recurso  .  "
	               + "   ?director <" +  ONTOLOGY_URI + "Nombre>  ?nombre .  " 
	               + " } ";
	       
	       QueryExecution qe = QueryExecutionFactory.create(query, iM);
	       
	       try {
	    	   ResultSet results = qe.execSelect();
	            // Para cada director encontrado
	            while(results.hasNext()){
	                QuerySolution r = results.nextSolution();
	                RDFNode director = r.get("nombre");
	                // Obtenemos nombre en formato String y lo aadimos a la lista
	                lista.add(((Literal)director).getLexicalForm()) ;
	            }
	        } catch (Exception ex) {
	            ex.printStackTrace();
	        } finally {
				// Importante - liberar recursos despues de hacer la consulta
				qe.close();
			}
		
		return lista;
		
	}	

	public List<String> consultaProductor(InfModel iM, String titulo){
		 List<String> lista = new ArrayList<String>();
		 String query = PREFIX + 
	    		   " SELECT distinct ?nombre "
	               + " WHERE {  "
	               + "   ?recurso rdf:type <http://www.sfernandezve.org/cine#Pelicula> .  "
				   + "   ?recurso <" +  ONTOLOGY_URI + "Titulo>\""  + titulo +   "\"  .  "
				   + "   ?productor <" +  ONTOLOGY_URI + "esProductorDe> ?recurso  .  "
	               + "   ?productor <" +  ONTOLOGY_URI + "Nombre>  ?nombre .  " 
	               + " } ";
	       
	       QueryExecution qe = QueryExecutionFactory.create(query, iM);
	       
	       try {
	    	   ResultSet results = qe.execSelect();
	            // Para cada productor encontrado
	            while(results.hasNext()){
	                QuerySolution r = results.nextSolution();
	                RDFNode productor = r.get("nombre");
	                // Obtenemos nombre en formato String y lo aadimos a la lista
	                lista.add(((Literal)productor).getLexicalForm()) ;
	            }
	        } catch (Exception ex) {
	            ex.printStackTrace();
	        } finally {
				// Importante - liberar recursos despues de hacer la consulta
				qe.close();
			}
		
		return lista;
		
	}		

	public List<String> consultaProtagonista(InfModel iM, String titulo){
		 List<String> lista = new ArrayList<String>();
		 String query = PREFIX + 
	    		   " SELECT distinct ?nombre "
	               + " WHERE {  "
	               + "   ?recurso rdf:type <http://www.sfernandezve.org/cine#Pelicula> .  "
				   + "   ?recurso <" +  ONTOLOGY_URI + "Titulo>\""  + titulo +   "\"  .  "
				   + "   ?protagonista <" +  ONTOLOGY_URI + "esProtagonistaDe> ?recurso .  "
	               + "   ?protagonista <" +  ONTOLOGY_URI + "Nombre>  ?nombre .  " 
	               + " } ";
	       
	       QueryExecution qe = QueryExecutionFactory.create(query, iM);
	       
	       try {
	    	   ResultSet results = qe.execSelect();
	            // Para cada protagonista encontrado
	            while(results.hasNext()){
	                QuerySolution r = results.nextSolution();
	                RDFNode protagonista = r.get("nombre");
	                // Obtenemos nombre en formato String y lo aadimos a la lista
	                lista.add(((Literal)protagonista).getLexicalForm()) ;
	            }
	        } catch (Exception ex) {
	            ex.printStackTrace();
	        } finally {
				// Importante - liberar recursos despues de hacer la consulta
				qe.close();
			}
		
		return lista;
		
	}	


	public List<String> consultaPeliculasDirigidas(InfModel iM, String nombre){
		 List<String> lista = new ArrayList<String>();
		 String query = PREFIX + 
	    		   " SELECT distinct ?titulo "
	               + " WHERE {  "
	               + "   ?recurso rdf:type <http://www.sfernandezve.org/cine#Director> .  "
	               + "   ?recurso <" +  ONTOLOGY_URI + "Nombre>\"" + nombre + "\" .  "
	               + "   ?recurso <" +  ONTOLOGY_URI + "esDirectorDe> ?pelicula .  "
	               + "   ?pelicula <" +  ONTOLOGY_URI + "Titulo> ?titulo .  "
	               + " } ";
	       
	       QueryExecution qe = QueryExecutionFactory.create(query, iM);
	       
	       try {
	    	   ResultSet results = qe.execSelect();
	            // Para cada pelicula encontrada
	            while(results.hasNext()){
	                QuerySolution r = results.nextSolution();
	                RDFNode titulo = r.get("titulo");
	                // Obtenemos nombre en formato String y lo aadimos a la lista
	                lista.add(((Literal)titulo).getLexicalForm()) ;
	            }
	        } catch (Exception ex) {
	            ex.printStackTrace();
	        } finally {
				// Importante - liberar recursos despues de hacer la consulta
				qe.close();
			}
		
		return lista;
		
	}	
	
	public List<String> consultaPeliculasProducidas(InfModel iM, String nombre){
		 List<String> lista = new ArrayList<String>();
		 String query = PREFIX + 
	    		   " SELECT distinct ?titulo "
	               + " WHERE {  "
	               + "   ?recurso rdf:type <http://www.sfernandezve.org/cine#Productor> .  "
	               + "   ?recurso <" +  ONTOLOGY_URI + "Nombre>\"" + nombre + "\" .  "
	               + "   ?recurso <" +  ONTOLOGY_URI + "esProductorDe> ?pelicula .  "
	               + "   ?pelicula <" +  ONTOLOGY_URI + "Titulo> ?titulo .  "
	               + " } ";
	       
	       QueryExecution qe = QueryExecutionFactory.create(query, iM);
	       
	       try {
	    	   ResultSet results = qe.execSelect();
	            // Para cada pelicula encontrada
	            while(results.hasNext()){
	                QuerySolution r = results.nextSolution();
	                RDFNode titulo = r.get("titulo");
	                // Obtenemos nombre en formato String y lo aadimos a la lista
	                lista.add(((Literal)titulo).getLexicalForm()) ;
	            }
	        } catch (Exception ex) {
	            ex.printStackTrace();
	        } finally {
				// Importante - liberar recursos despues de hacer la consulta
				qe.close();
			}
		
		return lista;
		
	}	
	
	public List<String> consultaPeliculasProtagonizadas(InfModel iM, String nombre){
		 List<String> lista = new ArrayList<String>();
		 String query = PREFIX + 
	    		   " SELECT distinct ?titulo "
	               + " WHERE {  "
	               + "   ?recurso rdf:type <http://www.sfernandezve.org/cine#Protagonista> .  "
	               + "   ?recurso <" +  ONTOLOGY_URI + "Nombre>\"" + nombre + "\" .  "
	               + "   ?recurso <" +  ONTOLOGY_URI + "esProtagonistaDe> ?pelicula .  "
	               + "   ?pelicula <" +  ONTOLOGY_URI + "Titulo> ?titulo .  "
	               + " } ";
	       
	       QueryExecution qe = QueryExecutionFactory.create(query, iM);
	       
	       try {
	    	   ResultSet results = qe.execSelect();
	            // Para cada pelicula encontrada
	            while(results.hasNext()){
	                QuerySolution r = results.nextSolution();
	                RDFNode titulo = r.get("titulo");
	                // Obtenemos nombre en formato String y lo aadimos a la lista
	                lista.add(((Literal)titulo).getLexicalForm()) ;
	            }
	        } catch (Exception ex) {
	            ex.printStackTrace();
	        } finally {
				// Importante - liberar recursos despues de hacer la consulta
				qe.close();
			}
		
		return lista;
		
	}	

	public List<String> extractDbpediaPeliculas(String director){
		
		List<String> lista = new ArrayList<String>();
		String queryString = PREFIX + 
				" SELECT ?titulo WHERE { " +
			           " ?titulo dbpedia1:director " + ":" + director + " . " +
			           " ?titulo a dbpedia1:Film . " +
			           " } ORDER BY DESC(?titulo) LIMIT 5 OFFSET 1 ";
	
		Query query = QueryFactory.create(queryString);

		// Ejecutar la query y obtener los resultados usando el SPARQL EndPoint de DBpedia 
		
		QueryExecution qe = QueryExecutionFactory.sparqlService("http://DBpedia.org/sparql",query);
		try {
			ResultSet results = qe.execSelect();

			// Procesar resultados
			
			for ( ; results.hasNext() ; )
			{
				QuerySolution solucion = results.nextSolution() ;  
				
				RDFNode variable = solucion.get("titulo");
				if (variable.isLiteral()) lista.add(((Literal)variable).getLexicalForm()) ;
				if (variable.isResource()) lista.add(((Resource)variable).getURI()) ;
				
			}
		} catch (Exception e) { 
			e.printStackTrace();
			
		} finally {
			// Importante - liberar recursos despues de hacer la consulta
			qe.close();
		}
		return lista;

	}
	
	public List<String> extractDbpediaProtagonistas(String pelicula){
			
			List<String> lista = new ArrayList<String>();
			String queryString = PREFIX + 
					" SELECT ?protagonista WHERE { " +
			          "<"  + pelicula + "> dbpedia1:starring  ?protagonista . " +
			           " } ORDER BY ASC(?protagonista)";
	
			
		Query query = QueryFactory.create(queryString);
	
		// Ejecutar la query y obtener los resultados usando el SPARQL EndPoint de DBpedia 
		
		QueryExecution qe = QueryExecutionFactory.sparqlService("http://DBpedia.org/sparql",query);
		try {
			ResultSet results = qe.execSelect();
	
			// Procesar resultados
			
			for ( ; results.hasNext() ; )
			{
				QuerySolution solucion = results.nextSolution() ;  
				
				RDFNode variable = solucion.get("protagonista");
				if (variable.isLiteral()) lista.add(((Literal)variable).getLexicalForm()) ;
				if (variable.isResource()) lista.add(((Resource)variable).getURI()) ;
				
			}
		} catch (Exception e) { 
			e.printStackTrace();
		} finally {
			// Importante - liberar recursos despues de hacer la consulta
			qe.close();
		}
		return lista;
	}
	
	public List<String> extractDbpediaProductores(String pelicula){
		
		List<String> lista = new ArrayList<String>();
		String queryString = PREFIX + 
				" SELECT ?productor WHERE { " +
		          "<" + pelicula + "> dbpedia1:producer   ?productor . " +
		           " } ORDER BY ASC(?productor)";

		Query query = QueryFactory.create(queryString);

		// Ejecutar la query y obtener los resultados usando el SPARQL EndPoint de DBpedia 
	
		QueryExecution qe = QueryExecutionFactory.sparqlService("http://DBpedia.org/sparql",query);
		try {
			ResultSet results = qe.execSelect();

			// Procesar resultados
		
			for ( ; results.hasNext() ; )
			{
				QuerySolution solucion = results.nextSolution() ;  
			
				RDFNode variable = solucion.get("productor");
				if (variable.isLiteral()) lista.add(((Literal)variable).getLexicalForm()) ;
				if (variable.isResource()) lista.add(((Resource)variable).getURI()) ;
			}
		} catch (Exception e) { 
			e.printStackTrace(); 
			
		} finally {
			// Importante - liberar recursos despues de hacer la consulta
			qe.close();
		}
		return lista;
	}
	
	public String extractDbpediaNombre(String persona){
			
		    String nombre = "Desconocido";
			String queryString = PREFIX + 
					" SELECT ?nombre WHERE { " +
			          "<" + persona + "> dbpedia2:name   ?nombre . " +
			           " } ORDER BY ASC(?nombre)";

			Query query = QueryFactory.create(queryString);
	
			// Ejecutar la query y obtener los resultados usando el SPARQL EndPoint de DBpedia 
		
			QueryExecution qe = QueryExecutionFactory.sparqlService("http://DBpedia.org/sparql",query);
			try {
				ResultSet results = qe.execSelect();
	
				// Procesar resultados
			
				if (results.hasNext())
				{
					QuerySolution solucion = results.nextSolution() ;  
				
					RDFNode variable = solucion.get("nombre");
					if (variable.isLiteral())  nombre = ((Literal)variable).getLexicalForm() ;
					if (variable.isResource()) nombre = (((Resource)variable).getURI()) ;
				}
			} catch (Exception e) { 
				e.printStackTrace(); 
				
			} finally {
				// Importante - liberar recursos despues de hacer la consulta
				qe.close();
			}
			return nombre;
		}
	
	public String extractDbpediaTitulo(String pelicula){
		
	    String titulo = "Desconocido";
		String queryString = PREFIX + 
				" SELECT ?titulo WHERE { " +
		          "<" + pelicula + "> dbpedia2:name   ?titulo . " +
		           " } ORDER BY ASC(?titulo)";

		Query query = QueryFactory.create(queryString);

		// Ejecutar la query y obtener los resultados usando el SPARQL EndPoint de DBpedia 
	
		QueryExecution qe = QueryExecutionFactory.sparqlService("http://DBpedia.org/sparql",query);
		try {
			ResultSet results = qe.execSelect();

			// Procesar resultados
		
			if (results.hasNext())
			{
				QuerySolution solucion = results.nextSolution() ;  
			
				RDFNode variable = solucion.get("titulo");
				if (variable.isLiteral())  titulo = ((Literal)variable).getLexicalForm() ;
				if (variable.isResource()) titulo = (((Resource)variable).getURI()) ;
			}
		} catch (Exception e) { 
			e.printStackTrace(); 
			
		} finally {
			// Importante - liberar recursos despues de hacer la consulta
			qe.close();
		}
		return titulo;
	}
	
	public String extractDbpediaSinopsis(String pelicula){
			
		    String sinopsis = "Desconocido";
			String queryString = PREFIX + 
					" SELECT ?sinopsis WHERE { " +
			          "<" + pelicula + "> dbpedia1:abstract   ?sinopsis . " +
					  " FILTER ( lang( ?sinopsis ) = \"en\" )"  +
			           " } ";
	
			Query query = QueryFactory.create(queryString);
	
			// Ejecutar la query y obtener los resultados usando el SPARQL EndPoint de DBpedia 
		
			QueryExecution qe = QueryExecutionFactory.sparqlService("http://DBpedia.org/sparql",query);
			try {
				ResultSet results = qe.execSelect();
	
				// Procesar resultados
			
				if (results.hasNext())
				{
					QuerySolution solucion = results.nextSolution() ;  
				
					RDFNode variable = solucion.get("sinopsis");
					if (variable.isLiteral())  sinopsis = ((Literal)variable).getLexicalForm() ;
					if (variable.isResource()) sinopsis = (((Resource)variable).getURI()) ;
				}
			} catch (Exception e) { 
				e.printStackTrace(); 
				
			} finally {
				// Importante - liberar recursos despues de hacer la consulta
				qe.close();
			}
			return sinopsis;
		}
	
	public void listarTodasSentencias(Model m, Resource s, Property p, Resource o) {
		for (StmtIterator i = m.listStatements(s,p,o); i.hasNext(); ) {
			Statement stmt = i.nextStatement();
			System.out.println(" - " + PrintUtil.print(stmt));
		}
	}
	
	public void cerrarConexion(){
		try {
			conn.close();
		} catch(Exception e) { 
			e.printStackTrace(); 
		}

	}
}


